/* Funcoes graficas basicas com acesso direto ao video.

Assembler AT&T + DJGPP:

- se gravar em variavel de memoria passada pelos parametros
  por "memory" no 3 :;
- __volatile__ permite ao compilador reorganizar o codigo */



#include <go32.h>
#include <string.h>   // necessario para acessar sys/movedata.h
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <conio.h>
#include <pc.h>

#include "defs.h"

#include "grvd.h"



/* construtor - inicia modo grafico, programa palette.
bInicGrVd == TRUE (default) inicia modo grafico, reprograma palette.
bInicGrVd altera comportamento do destrutor.
**********************************************************/
CGrVd::CGrVd ( BOOL bInicGrVd )
{
   // define como serah iniciada a classe
   m_bInicGrVd = bInicGrVd;

   // verifica se deve entrar em modo grafico e reprogramar palette
   if ( m_bInicGrVd == TRUE ) {

      // entra em modo grafico
      mAtrVdModo ( MCGA );

      // programa palette
      for ( INT ncC = 0; ncC < 64; ncC ++ ) {

         mAtrPalCor ( ncC +   0,   0,   0, ncC );

         mAtrPalCor ( ncC +  64, ncC,   0,  64 );

         mAtrPalCor ( ncC + 128, 64 - ncC,   32, 64 - ncC );

         mAtrPalCor ( ncC + 192,   0, ncC,  64 );
      }

      // programa palette
      mAtrPalCor ( 7, 55, 0, 0 );
   }
}



/* destrutor - Volta ao modo texto, desaloca memoria
**********************************************************/
CGrVd::~CGrVd ()
{
   // verifica se entrou em modo grafico no inicio da classe
   if ( m_bInicGrVd == TRUE )
      // entra em modo texto
      mAtrVdModo ( TEXTO );
}



/* Altera o modo de exibicao do video.
**********************************************************/
void CGrVd::mAtrVdModo ( UINT16 uModo )
{
   REGS
      stRegsE, stRegsS;      // registradores

   stRegsE.x.ax = uModo;
   int86 ( 0x10, &stRegsE, &stRegsS );

   // se entra em modo MCGA altera proporcao entre x e y para 1x1
   if ( uModo == MCGA )
      outportb ( 0x3C2, 0xE3 );
}



/* Atribui uma quantidade de RGB para uma cor (entrada) da palette
**********************************************************/
void CGrVd::mAtrPalCor ( BYTE yCor, BYTE yR, BYTE yG, BYTE yB )
{
   // solicita alteracao de palette
   outportb ( 0x03C8, yCor );
   // altera
   outportb ( 0x03C9, yR );
   outportb ( 0x03C9, yG );
   outportb ( 0x03C9, yB );
}



/* Retorna os componentes RGB de uma cor (entrada) da palette
**********************************************************/
void CGrVd::mRetPalCor ( BYTE yCor, BYTE& yR, BYTE& yG, BYTE& yB )
{
   // solicita leitura de palette
   outportb ( 0x03C7, yCor );
   // le
   yR = inportb ( 0x03C9 );
   yG = inportb ( 0x03C9 );
   yB = inportb ( 0x03C9 );
}



/* Guarda em tabelas internas os componentes RGB de todas as
 cores da palette. Usado para retaurar palette apohs alteracoes
 com CGrVd::mRestauraPal
**********************************************************/
void CGrVd::mSalvaPal ()
{
   // varre toda palette lendo componentes de cada cor
   for ( UINT16 ucCor = 0; ucCor < 256; ucCor ++ )
      mRetPalCor ( ucCor, m_ayPalR [ ucCor ], m_ayPalG [ ucCor ], m_ayPalB [ ucCor ] );
}



/* Altera a palette com as cores das tabelas internas. Usado em
conjunto com CGrVd::mSalvaPal
**********************************************************/
void CGrVd::mRestauraPal ()
{
   // varre toda palette alterando componentes de cada cor
   for ( UINT16 ucCor = 0; ucCor < 256; ucCor ++ )
      mAtrPalCor ( ucCor, m_ayPalR [ ucCor ], m_ayPalG [ ucCor ], m_ayPalB [ ucCor ] );
}



/* Programa a palette conforme componentes RGB passadas por uma tabela
 cada 3 bytes da tabelainformam uma cor.
**********************************************************/
void CGrVd::mAtrPal ( BYTE* ayPalRGB )
{
   // varre toda palette alterando componentes de cada cor
   for ( UINT16 ucCor = 0; ucCor < 768; ucCor += 3 )
      mAtrPalCor ( ucCor / 3, * ( ayPalRGB + ucCor ), * ( ayPalRGB + ucCor + 1 ), * ( ayPalRGB + ucCor + 2 ) );
}



/* Espera retraco de video para desenhar (=="WaitRetrace"),
**********************************************************/
void CGrVd::mVdSincRetraco ()
{
   __asm__ __volatile__ ("

          movw  $0x03DA, %%dx
      L1: inb   %%dx,    %%al
          andb  $0x8,    %%al
          jnz   L1

      L2: inb   %%dx,    %%al
          andb  $0x8,    %%al
          jz    L2
      "
      :
      :
      : "%dx", "%al"
   );
}



/* Atribui toda a tela com uma soh cor. Sincroniza com retraco.
**********************************************************/
void CGrVd::mAtrVdCorSinc ( BYTE yCor )
{
   BYTE
      ayCores [ 4 ];
   ayCores [ 0 ] = ayCores [ 1 ] = ayCores [ 2 ] = ayCores [ 3 ] = yCor;

   mVdSincRetraco ();

   for ( INT ncTrans = 0; ncTrans < TAMVR; ncTrans += 4 )
      _dosmemputl ( (void*) ayCores, 1, 0xA0000 + ncTrans );
}



/* Marca um pixel na tela
**********************************************************/
void CGrVd::mAtrVdPixel ( INT nX, INT nY, BYTE yCor )
{
   dosmemput ( (void*) &yCor, 1, 0xA0000 + nX + ( nY * RESX ) );
}
